import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React, { useCallback, useContext } from 'react';
import cx from 'classnames';
import { bool, object, string, func, oneOfType, shape, elementType } from 'prop-types';
import invariant from 'invariant';
import RRContext from './ReactRouterContext';

var isLeftClickEvent = function isLeftClickEvent(event) {
  return event.button === 0;
};

var isModifiedEvent = function isModifiedEvent(event) {
  return Boolean(event.metaKey || event.altKey || event.ctrlKey || event.shiftKey);
};

var resolveToLocation = function resolveToLocation(to, router) {
  return typeof to === 'function' ? to(router.location) : to;
};

var isEmptyObject = function isEmptyObject(object) {
  if (!object) {
    return true;
  }

  for (var p in object) {
    if (Object.prototype.hasOwnProperty.call(object, p)) return false;
  }

  return true;
};

var Link = function Link(_ref) {
  var to = _ref.to,
      onClick = _ref.onClick,
      target = _ref.target,
      activeClassName = _ref.activeClassName,
      activeStyle = _ref.activeStyle,
      innerRef = _ref.innerRef,
      _ref$onlyActiveOnInde = _ref.onlyActiveOnIndex,
      onlyActiveOnIndex = _ref$onlyActiveOnInde === void 0 ? false : _ref$onlyActiveOnInde,
      props = _objectWithoutPropertiesLoose(_ref, ["to", "onClick", "target", "activeClassName", "activeStyle", "innerRef", "onlyActiveOnIndex"]);

  var _useContext = useContext(RRContext),
      router = _useContext.router;

  var handleClick = useCallback(function (event) {
    if (onClick) onClick(event);
    if (event.defaultPrevented) return;
    if (isModifiedEvent(event) || !isLeftClickEvent(event)) return; // If target prop is set (e.g. to "_blank"), let browser handle link.

    if (target) return;
    event.preventDefault();
    router.push(resolveToLocation(to, router));
  }, [onClick, router, target, to]);

  if (!router) {
    invariant(router, '<Link>s rendered outside of a router context cannot navigate.');
    return null;
  }

  if (!to) {
    return React.createElement("a", Object.assign({}, props, {
      onClick: onClick,
      ref: innerRef
    }));
  }

  var toLocation = resolveToLocation(to, router);
  props = _extends({}, props, {
    href: router.createHref(toLocation)
  });

  if (activeClassName) {
    var _cx;

    props.className = cx(props.className, (_cx = {}, _cx[activeClassName] = router.isActive(toLocation, onlyActiveOnIndex), _cx));
  }

  if (!isEmptyObject(activeStyle)) props.style = _extends({}, props.style, {}, activeStyle);
  return React.createElement("a", Object.assign({
    onClick: handleClick,
    target: target,
    ref: innerRef
  }, props));
};

export default Link;