import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import React, { useState, useEffect } from 'react';
import { func, object } from 'prop-types';
import { routes, component, components } from './InternalPropTypes';
import { createTransitionManager } from './createTransitionManager';
import { fromReactTreeToJSON } from '../lib/utils/react-utils';
import { createRouterObject } from './utils/RouterUtils';
import RRContext from './ReactRouterContext';

var Router = function Router(_ref) {
  var _ref2, _ref3;

  var components = _ref.components,
      children = _ref.children,
      routes = _ref.routes,
      render = _ref.render,
      matchContext = _ref.matchContext,
      params = _ref.params,
      history = _ref.history,
      onError = _ref.onError,
      props = _objectWithoutPropertiesLoose(_ref, ["components", "children", "routes", "render", "matchContext", "params", "history", "onError"]);

  var transitionManager = (_ref2 = matchContext == null ? void 0 : matchContext.transitionManager) != null ? _ref2 : createTransitionManager({
    history: history,
    jsonRoutes: fromReactTreeToJSON(routes)
  });
  var router = (_ref3 = props == null ? void 0 : props.router) != null ? _ref3 : createRouterObject(history, transitionManager, {
    location: null,
    routes: null,
    params: null
  });

  var _useState = useState({
    router: router,
    params: params,
    components: components
  }),
      state = _useState[0],
      setState = _useState[1];

  useEffect(function () {
    var unlisten = matchContext.transitionManager.listen(function (err, nextState) {
      if (err) {
        if (onError) {
          return onError(err);
        }

        throw err;
      }

      var params = nextState.params,
          components = nextState.components,
          location = nextState.location;

      var nextRouter = _extends({}, state.router, {
        params: params,
        location: location,
        routes: routes
      });

      setState({
        router: nextRouter,
        params: params,
        components: components
      });
    });
    return function () {
      return unlisten();
    };
  }, []); // eslint-disable-line

  return React.createElement(RRContext.Provider, {
    value: {
      router: state.router,
      params: state.params
    }
  }, render({
    components: state.components,
    params: state.params,
    router: state.router,
    props: props
  }));
};

Router.displayName = 'Router';
Router.defaultProps = {
  render: function render(_ref4) {
    var components = _ref4.components,
        params = _ref4.params,
        router = _ref4.router,
        props = _ref4.props;

    var pipeReact = function pipeReact(components) {
      return function (base) {
        return components.reduceRight(function (acc, component) {
          return React.createElement(component, _extends({}, props, {
            params: params,
            router: router,
            children: acc
          }));
        }, base);
      };
    };

    var tree = pipeReact(components)(null);
    return tree;
  }
};
export default Router;