// https://jrsinclair.com/articles/2019/functional-js-traversing-trees-with-recursive-reduce/
var curry = function curry(fn, arity) {
  if (arity === void 0) {
    arity = fn.length;
  }

  for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  return arity <= args.length ? fn.apply(void 0, args) : curry.bind.apply(curry, [null, fn, arity].concat(args));
};

function hasChildren(node) {
  return typeof node === 'object' && typeof node.children !== 'undefined' && node.children.length > 0;
}

export var Tree = {
  reduce: curry(function reduce(reducerFn, init, node) {
    var acc = reducerFn(init, node);

    if (!hasChildren(node)) {
      return acc;
    }

    return node.children.reduce(Tree.reduce(reducerFn), acc);
  }),
  map: curry(function map(mapFn, node) {
    var newNode = mapFn(node);

    if (hasChildren(node)) {
      return newNode;
    }

    newNode.children = node.children.map(Tree.map(mapFn));
    return newNode;
  }),
  forEach: curry(function forEach(eachFn, node) {
    eachFn(node);

    if (hasChildren(node)) {
      node.children.forEach(Tree.forEach(eachFn));
    }
  }),
  tap: function tap(tree) {
    Tree.forEach(function (node) {
      console.log(node); // eslint-disable-line
    }, tree);
  }
};